/* (C) 2021 YiRing, Inc. */
package com.yiring.app.config;

import cn.dev33.satoken.exception.NotLoginException;
import com.yiring.app.constant.Code;
import com.yiring.app.exception.CodeException;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.exception.FailStatusException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import lombok.extern.slf4j.Slf4j;
import org.apache.catalina.connector.ClientAbortException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * 全局错误处理
 *
 * @author ifzm
 * 2017年11月30日 上午11:36:31
 */
@Slf4j
@ControllerAdvice
@ResponseBody
public class GlobalExceptionHandler {

    /**
     * 参数校验异常
     *
     * @param e 异常信息
     * @return 统一的校验失败信息 {@link Status#EXPECTATION_FAILED
     */
    @ExceptionHandler(
        value = { BindException.class, MethodArgumentNotValidException.class, ConstraintViolationException.class }
    )
    public Result<String> bindErrorHandler(Exception e) {
        String error = "未知参数校验错误";

        if (e instanceof ConstraintViolationException) {
            error = ((ConstraintViolationException) e).getConstraintViolations().iterator().next().getMessage();
        } else {
            BindingResult result = null;
            if (e instanceof MethodArgumentNotValidException) {
                result = ((MethodArgumentNotValidException) e).getBindingResult();
            } else if (e instanceof BindException) {
                result = ((BindException) e).getBindingResult();
            }

            if (result != null) {
                error = result.getAllErrors().iterator().next().getDefaultMessage();
            }
        }

        return Result.no(Status.EXPECTATION_FAILED, error);
    }

    /**
     * 不支持的HttpMethod异常
     *
     * @param e 异常信息
     * @return 异常信息反馈 {@link Status#METHOD_NOT_ALLOWED
     */
    @ExceptionHandler(value = HttpRequestMethodNotSupportedException.class)
    public Result<String> httpRequestMethodNotSupportedErrorHandler(Exception e) {
        return Result.no(Status.METHOD_NOT_ALLOWED, e.getMessage());
    }

    /**
     * 未登录异常（鉴权失败）
     *
     * @return 异常信息反馈 {@link Status#UNAUTHORIZED
     */
    @ExceptionHandler(value = NotLoginException.class)
    public Result<String> notLoginErrorHandler() {
        return Result.no(Status.UNAUTHORIZED);
    }

    /**
     * 自定义业务异常
     */
    @ExceptionHandler(value = CodeException.class)
    public Result<String> customCodeExceptionHandler(CodeException e) {
        Code code = e.getCode();
        return Result.no(Status.BAD_REQUEST, code.value(), code.reason(), null);
    }

    /**
     * 失败状态异常
     */
    @ExceptionHandler(value = FailStatusException.class)
    public Result<String> failStatusExceptionHandler(FailStatusException e) {
        return Result.no(e.getStatus());
    }

    /**
     * 取消请求异常（忽略）
     */
    @ExceptionHandler(value = ClientAbortException.class)
    public void clientAbortExceptionHandler() {}

    /**
     * 其他异常
     *
     * @param e 异常信息
     * @return 统一的500异常信息 {@link Status#INTERNAL_SERVER_ERROR
     */
    @ExceptionHandler(value = Exception.class)
    public Result<String> defaultErrorHandler(Exception e, HttpServletResponse response) {
        response.setStatus(Status.INTERNAL_SERVER_ERROR.value());
        log.error(e.getMessage(), e);
        return Result.no(Status.INTERNAL_SERVER_ERROR, e);
    }
}
