/* (C) 2022 YiRing, Inc. */
package com.yiring.app.job;

import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.yiring.app.domain.location.LocationLog;
import com.yiring.app.domain.location.LocationLogRepository;
import com.yiring.app.rabbit.config.ZyRabbitConfig;
import com.yiring.app.util.GeoUtils;
import com.yiring.common.constant.DateFormatter;
import java.time.LocalDateTime;
import java.util.List;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Point;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

/**
 * 模拟真源消息日志推送
 *
 * @author Jim
 * @version 0.1
 * 2022/5/7 15:26
 */

@SuppressWarnings("unused")
@Slf4j
@Component
public class MockZyMessageJob {

    @Resource
    RabbitTemplate rabbitTemplate;

    @Resource
    LocationLogRepository locationLogRepository;

    @XxlJob("MockZyMessageHandler")
    public void mockMessageHandler() {
        log.info("MockZyMessageHandler: {}", LocalDateTime.now().format(DateFormatter.DATE_TIME));

        log.info("[Mock] Position: {}", mockPositionMessage());
        log.info("[Mock] LowPower: {}", mockLowPowerMessage());
        log.info("[Mock] DeviceStatus: {}", mockDeviceStatusMessage());
        log.info("[Mock] KeyWarning: {}", mockKeyWarningMessage());
    }

    @XxlJob("QueryMessageHandler")
    public void queryMessageHandler() {
        log.info("QueryZyMessageHandler: {}", LocalDateTime.now().format(DateFormatter.DATE_TIME));

        try {
            Specification<LocationLog> spec = (root, query, cb) -> {
                Predicate predicate = cb.conjunction();
                predicate.getExpressions().add(cb.lessThanOrEqualTo(root.get("id").get("time"), LocalDateTime.now()));
                return predicate;
            };

            List<LocationLog> logs = locationLogRepository.findAll(spec);
            log.info("QueryZyMessageHandler: {}", logs.size());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public JSONObject send(JSONObject body) {
        rabbitTemplate.convertAndSend(ZyRabbitConfig.MESSAGE_QUEUES_NAME, body.toJSONString());
        return body;
    }

    private String mockTag() {
        return "BTT33333331";
    }

    private Long mockAreaId() {
        return 10019L;
    }

    private JSONObject mockPositionMessage() {
        // 随机生成一个坐标点
        Point point = GeoUtils.randomPoint(GeoUtils.defaultBounds(), 0);

        // 模拟定位数据
        JSONObject params = new JSONObject();
        params.put("tagId", mockTag());
        params.put("areaId", mockAreaId());
        params.put("silent", false);
        params.put("beacons", "BTI22085237,BTI22085238");
        params.put("longitude", point.getCoordinate().getX());
        params.put("latitude", point.getCoordinate().getY());
        params.put("altitude", point.getCoordinate().getZ());
        params.put("locationTime", System.currentTimeMillis());
        params.put("volt", 3650);
        params.put("voltUnit", "mV");
        params.put("floor", 1);

        JSONObject body = new JSONObject();
        body.put("method", "position");
        body.put("params", params);
        return send(body);
    }

    private JSONObject mockLowPowerMessage() {
        JSONObject params = new JSONObject();
        params.put("tagId", mockTag());
        params.put("volt", 3650);
        params.put("voltUnit", "mV");

        JSONObject body = new JSONObject();
        body.put("method", "lowPower");
        body.put("params", params);
        return send(body);
    }

    private JSONObject mockDeviceStatusMessage() {
        JSONObject params = new JSONObject();
        params.put("deviceId", mockTag());
        params.put("areaId", mockAreaId());
        params.put("deviceType", "BTI");
        params.put("volt", 3650);
        params.put("field_21", "mV");
        params.put("updateTime", System.currentTimeMillis());

        JSONObject body = new JSONObject();
        body.put("method", "deviceStatus");
        body.put("params", params);
        return send(body);
    }

    private JSONObject mockKeyWarningMessage() {
        JSONObject params = new JSONObject();
        params.put("tagId", mockTag());
        params.put("entityId", "1522770547178475520");
        params.put("areaId", mockAreaId());
        params.put("raiseTime", System.currentTimeMillis());
        params.put("x", 100);
        params.put("y", 100);
        params.put("z", 0);
        params.put("floor", 1);

        JSONObject body = new JSONObject();
        body.put("method", "keyWarning");
        body.put("params", params);
        return send(body);
    }
}
