/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.alarm.impl;

import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.domain.alarm.AlarmTypeRepository;
import com.yiring.app.excel.alarm.AlarmTypeExcel;
import com.yiring.app.param.alarm.AlarmConditionParam;
import com.yiring.app.param.alarm.AlarmTypeAddParam;
import com.yiring.app.param.alarm.AlarmTypeModifyParam;
import com.yiring.app.service.alarm.AlarmService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.alarm.AlarmTypeVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/4/27 14:39
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class AlarmServiceImpl implements AlarmService {

    @Resource
    private AlarmTypeRepository alarmTypeRepository;

    @Override
    public Result<String> addAlarmType(AlarmTypeAddParam param) {
        Boolean exists = alarmTypeRepository.existsByName(param.getName());
        if (exists) {
            return Result.no(Status.EXPECTATION_FAILED, "名称已被占用");
        }
        alarmTypeRepository.saveAndFlush(param.transform());
        return Result.ok();
    }

    @Override
    public Result<String> modifyAlarmType(AlarmTypeModifyParam param) {
        AlarmType alarmType = null;
        List<AlarmType> allByName = alarmTypeRepository.findAllByName(param.getName());
        if (allByName.size() > 1) {
            log.warn("AlarmServiceImpl.modifyAlarmType时查询出两条报警类型名称相同的数据,param:[{}]", param);
        }
        for (AlarmType item : allByName) {
            if (item.getId().equals(param.getId())) {
                alarmType = item;
            } else {
                return Result.no(Status.EXPECTATION_FAILED, "名称已被占用");
            }
        }
        if (alarmType == null) {
            Optional<AlarmType> optional = alarmTypeRepository.findById(param.getId());
            if (optional.isEmpty()) {
                return Result.no(Status.EXPECTATION_FAILED, "修改的报警类型不存在");
            }
            alarmType = optional.get();
        }
        JpaUtil.copyNotNullProperties(param, alarmType);
        return Result.ok();
    }

    @Override
    public Result<String> removeAlarmType(IdParam param) {
        Optional<AlarmType> optional = alarmTypeRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "删除的报警类型不存在");
        }
        optional.get().setDeleted(true);
        return Result.ok();
    }

    @Override
    public Result<AlarmTypeVo> findById(IdParam param) {
        Optional<AlarmType> optional = alarmTypeRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "查询的报警类型不存在");
        }
        AlarmTypeVo vo = AlarmTypeVo.transform(optional.get());
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<AlarmTypeVo>> findByCondition(AlarmConditionParam conditionParam, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<AlarmType> specification = getSpecification(conditionParam);
        Page<AlarmType> page = alarmTypeRepository.findAll(specification, pageable);
        List<AlarmTypeVo> list = page.get().map(AlarmTypeVo::transform).collect(Collectors.toList());
        PageVo<AlarmTypeVo> pageVo = PageVo.build(list, page.getTotalElements());
        return Result.ok(pageVo);
    }

    @Override
    public void exportAlarmType(AlarmConditionParam conditionParam, HttpServletResponse response, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<AlarmType> specification = getSpecification(conditionParam);
        Page<AlarmType> page = alarmTypeRepository.findAll(specification, pageable);
        List<AlarmTypeExcel> list = page.get().map(AlarmTypeExcel::transform).collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<AlarmTypeExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(AlarmTypeExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(list);

            String fileName = URLEncoder.encode("报警类型列表.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("报警类型列表导出失败: " + e.getMessage());
        }
    }

    private Specification<AlarmType> getSpecification(AlarmConditionParam conditionParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (conditionParam.getFenceType() != null) {
                list.add(criteriaBuilder.equal(root.get("fenceType"), conditionParam.getFenceType()));
            }
            if (conditionParam.getRelevanceParam() != null) {
                list.add(criteriaBuilder.equal(root.get("relevanceParam"), conditionParam.getRelevanceParam()));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
    }
}
