/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.analysis.history.impl;

import com.alibaba.fastjson.JSONArray;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.constant.RedisKey;
import com.yiring.app.domain.location.LocationLog;
import com.yiring.app.domain.location.LocationLogRepository;
import com.yiring.app.excel.analysis.history.HistoryRouteExcel;
import com.yiring.app.param.analysis.history.HistoryRouteConditionParam;
import com.yiring.app.service.analysis.history.HistoryRouteService;
import com.yiring.app.vo.analysis.history.HistoryRouteVo;
import com.yiring.app.vo.map.MapVo;
import com.yiring.common.core.Redis;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/6 14:41
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class HistoryRouteServiceImpl implements HistoryRouteService {

    @Resource
    private LocationLogRepository locationLogRepository;

    @Resource
    private Redis redis;

    @Override
    public Result<PageVo<HistoryRouteVo>> findByCondition(HistoryRouteConditionParam param) {
        Specification<LocationLog> specification = getSpecification(param);
        ArrayList<MapVo.MapVoReuslt> list1 = new ArrayList<>();
        Object map = redis.get(RedisKey.ZY_MAP_AREAS);
        String mapString = JSONArray.toJSONString(map);
        List<MapVo.MapVoReuslt> mapVo = JSONArray.parseArray(mapString, MapVo.MapVoReuslt.class);
        List<LocationLog> list = locationLogRepository.findAll(specification);
        List<HistoryRouteVo> voList = list
            .stream()
            .map(e -> {
                HistoryRouteVo routeVo = HistoryRouteVo.transform(e);
                for (MapVo.MapVoReuslt item : mapVo) {
                    if (item.getId().equals(e.getAreaId().intValue())) {
                        routeVo.setMapName(item.getName());
                        break;
                    }
                }
                return routeVo;
            })
            .collect(Collectors.toList());
        PageVo<HistoryRouteVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public void exportHistoryRoute(HistoryRouteConditionParam param, HttpServletResponse response) {
        Result<PageVo<HistoryRouteVo>> result = findByCondition(param);
        if (Status.OK.value() != result.getStatus()) {
            return;
        }
        List<HistoryRouteExcel> excelList = result
            .getBody()
            .getData()
            .stream()
            .map(HistoryRouteExcel::transform)
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<HistoryRouteExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(HistoryRouteExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(excelList);

            String fileName = URLEncoder.encode("历史轨迹表.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("报警类型列表导出失败: " + e.getMessage());
        }
    }

    private Specification<LocationLog> getSpecification(HistoryRouteConditionParam param) {
        return (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (param.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get("areaId"), param.getMapId()));
            }
            if (param.getUserId() != null) {
                list.add(criteriaBuilder.equal(root.get("user"), param.getUserId()));
            }
            if (param.getBeginTime() != null) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("locationTime"), param.getBeginTime()));
            }
            if (param.getEndTime() != null) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("locationTime"), param.getEndTime()));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
    }
}
