/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.broadcast.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.domain.broadcast.Broadcast;
import com.yiring.app.domain.broadcast.BroadcastRepository;
import com.yiring.app.excel.broadcast.BroadcastExportExcel;
import com.yiring.app.excel.broadcast.BroadcastImportExcel;
import com.yiring.app.param.broadcast.BroadcastParam;
import com.yiring.app.param.broadcast.BroadcastQueryParam;
import com.yiring.app.service.broadcast.BroadcastService;
import com.yiring.app.util.GeoUtils;
import com.yiring.app.vo.broadcast.BroadcastVo;
import com.yiring.app.vo.broadcast.BroadcastjcVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.IndexParam;
import com.yiring.common.vo.PageVo;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.locationtech.jts.geom.Point;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author tzl
 * @version 1.0
 * @description:
 * @date 2022/5/5 17:32
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class BroadcastServiceImpl implements BroadcastService {

    @Resource
    BroadcastRepository broadcastRepository;

    @Override
    public Result<String> saveBroadcast(BroadcastParam broadcastParam) {
        if (has(broadcastParam.getBroadcastNum())) {
            return Result.no(Status.BAD_REQUEST, "播报设备编号已存在");
        }
        Broadcast broadcast = new Broadcast();
        BeanUtils.copyProperties(broadcastParam, broadcast);
        broadcast.setPoint(GeoUtils.createPoint(broadcastParam.getLon(), broadcastParam.getLat()));
        broadcastRepository.saveAndFlush(broadcast);
        return Result.ok();
    }

    @Override
    public Result<String> updateBroadcast(BroadcastParam broadcastParam, IdParam idParam) {
        Optional<Broadcast> byId = broadcastRepository.findById(idParam.getId());
        if (byId.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Broadcast broadcast = byId.get();
        if (!broadcast.getBroadcastNum().equals(broadcastParam.getBroadcastNum())) {
            if (has(broadcastParam.getBroadcastNum())) {
                return Result.no(Status.BAD_REQUEST, "播报设备编号已存在");
            }
        }
        BeanUtils.copyProperties(broadcastParam, broadcast);
        broadcast.setPoint(GeoUtils.createPoint(broadcastParam.getLon(), broadcastParam.getLat()));
        broadcastRepository.saveAndFlush(broadcast);
        return Result.ok();
    }

    @Override
    public Result<String> deleteBroadcast(IdParam idParam) {
        Optional<Broadcast> byId = broadcastRepository.findById(idParam.getId());
        if (byId.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Broadcast broadcast = byId.get();
        broadcastRepository.delete(broadcast);
        return Result.ok();
    }

    @Override
    public Result<Point> selectPoint(IdParam idParam) {
        Optional<Broadcast> byId = broadcastRepository.findById(idParam.getId());
        if (byId.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Broadcast broadcast = byId.get();
        return Result.ok(broadcast.getPoint());
    }

    @Override
    public Result<PageVo<BroadcastVo>> pageBroadcast(BroadcastQueryParam broadcastQueryParam) {
        PageVo<BroadcastVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Broadcast.Fields.updateTime));
        if (Objects.nonNull(broadcastQueryParam.getPageNo()) && Objects.nonNull(broadcastQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(broadcastQueryParam.getPageNo() - 1, broadcastQueryParam.getPageSize());
            Page<Broadcast> all = broadcastRepository.findAll(condition(broadcastQueryParam), pageable);
            List<BroadcastVo> data = all
                .get()
                .map(invitation -> {
                    BroadcastVo vo = new BroadcastVo();
                    BeanUtils.copyProperties(invitation, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Broadcast> all = broadcastRepository.findAll(condition(broadcastQueryParam), sort);
            List<BroadcastVo> data = all
                .stream()
                .map(invitation -> {
                    BroadcastVo vo = new BroadcastVo();
                    BeanUtils.copyProperties(invitation, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public Result<String> importBroadcast(MultipartFile file) {
        if (file.isEmpty()) {
            throw new RuntimeException("请选择文件");
        }
        List<BroadcastImportExcel> broadcastImportExcels;
        try (InputStream inputStream = file.getInputStream()) {
            broadcastImportExcels =
                SaxExcelReader
                    .of(BroadcastImportExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            throw new RuntimeException("文件导入异常");
        }

        List<Broadcast> broadcasts = broadcastImportExcels
            .stream()
            .map(broadcastImportExcel -> {
                Broadcast broadcast = new Broadcast();
                BeanUtils.copyProperties(broadcastImportExcel, broadcast);
                broadcast.setPoint(GeoUtils.createPoint(broadcastImportExcel.getLon(), broadcastImportExcel.getLat()));
                broadcast.setCreateTime(LocalDateTime.now());
                return broadcast;
            })
            .toList();
        //检查是否有编号重复的数据
        if (broadcasts != null) {
            broadcasts.forEach(broadcast ->
                broadcastRepository
                    .findOne((root, cq, cb) -> {
                        List<Predicate> predicates = new ArrayList<>();
                        if (StrUtil.isNotBlank(broadcast.getBroadcastNum())) {
                            predicates.add(
                                cb.equal(
                                    root.get(Broadcast.Fields.broadcastNum).as(String.class),
                                    broadcast.getBroadcastNum()
                                )
                            );
                        }
                        return cq.where(predicates.toArray(new Predicate[0])).getRestriction();
                    })
                    .ifPresent(one -> {
                        //如果存在则操作修改
                        broadcast.setId(one.getId());
                    })
            );
            //根据编号去重
            broadcasts =
                broadcasts
                    .stream()
                    .collect(
                        Collectors.collectingAndThen(
                            Collectors.toCollection(() ->
                                new TreeSet<>(Comparator.comparing(Broadcast::getBroadcastNum))
                            ),
                            ArrayList::new
                        )
                    );
            broadcastRepository.saveAllAndFlush(broadcasts);
        }
        return Result.ok();
    }

    @Override
    public Result<PageVo<BroadcastjcVo>> selectBroadcastName(IndexParam indexParam) {
        Sort sort = Sort.by(Sort.Order.desc(Broadcast.Fields.updateTime));
        List<Broadcast> all = broadcastRepository.findAll(
            condition(BroadcastQueryParam.builder().broadcastName(indexParam.getStr()).build()),
            sort
        );
        List<BroadcastjcVo> data = all
            .stream()
            .map(broadcast -> {
                BroadcastjcVo vo = new BroadcastjcVo();
                BeanUtils.copyProperties(broadcast, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<BroadcastjcVo> resultVo = PageVo.build(data, data.size());
        return Result.ok(resultVo);
    }

    @Override
    public void exportBroadcast(BroadcastQueryParam broadcastQueryParam, HttpServletResponse response) {
        List<Broadcast> broadcasts = broadcastRepository.findAll(condition(broadcastQueryParam));

        List<BroadcastExportExcel> broadcastExportExcels = broadcasts
            .stream()
            .map(broadcast -> {
                BroadcastExportExcel broadcastExportExcel = new BroadcastExportExcel();
                BeanUtils.copyProperties(broadcast, broadcastExportExcel);
                broadcastExportExcel.setLon(broadcast.getPoint().getX());
                broadcastExportExcel.setLat(broadcast.getPoint().getY());
                return broadcastExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<BroadcastExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(BroadcastExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(broadcastExportExcels);

            String fileName = URLEncoder.encode("播报信息信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出播报信息信息失败: " + e.getMessage());
        }
    }

    public Specification<Broadcast> condition(BroadcastQueryParam broadcastQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (StrUtil.isNotBlank(broadcastQueryParam.getBroadcastName())) {
                list.add(
                    criteriaBuilder.like(
                        root.get(Broadcast.Fields.broadcastName).as(String.class),
                        "%" + broadcastQueryParam.getBroadcastName() + "%"
                    )
                );
            }
            if (StrUtil.isNotBlank(broadcastQueryParam.getBroadcastNum())) {
                list.add(
                    criteriaBuilder.like(
                        root.get(Broadcast.Fields.broadcastNum).as(String.class),
                        "%" + broadcastQueryParam.getBroadcastNum() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(broadcastQueryParam.getStatus())) {
                list.add(
                    criteriaBuilder.like(
                        root.get(Broadcast.Fields.status).as(String.class),
                        "%" + broadcastQueryParam.getStatus() + "%"
                    )
                );
            }
            Order order = criteriaBuilder.desc(root.get(Broadcast.Fields.updateTime));
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            query.orderBy(order);
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }

    boolean has(String param) {
        Broadcast broadcast = Broadcast.builder().broadcastNum(param).build();
        return broadcastRepository.count(Example.of(broadcast)) > 0;
    }
}
