/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.dict.impl;

import cn.hutool.core.util.StrUtil;
import com.yiring.app.param.dict.DictParam;
import com.yiring.app.param.dict.DictQueryParam;
import com.yiring.app.service.dict.DictService;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.dict.Dict;
import com.yiring.common.dict.DictRepository;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.util.DictUtils;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

/**
 * 字典
 *
 * @author tzl
 * 2022/4/13 15:37
 */

@Service
public class DictServiceImpl implements DictService {

    @Resource
    DictRepository dictRepository;

    /**
     * 项目启动时，初始化字典到缓存
     */
    @PostConstruct
    public void init() {
        loadingDictCache();
    }

    /**
     * 加载字典缓存数据
     */
    @Override
    public void loadingDictCache() {
        Dict dict = Dict.builder().status("1").build();
        Map<String, List<Dict>> dictDataMap = dictRepository
            .findAll(Example.of(dict))
            .stream()
            .collect(Collectors.groupingBy(Dict::getDictType));
        for (Map.Entry<String, List<Dict>> entry : dictDataMap.entrySet()) {
            DictUtils.setDictCache(
                entry.getKey(),
                entry.getValue().stream().sorted(Comparator.comparing(Dict::getDictSort)).collect(Collectors.toList())
            );
        }
    }

    @Override
    public Result<PageVo<Dict>> pageDictInfo(DictQueryParam dictQueryParam, PageParam param) {
        Page<Dict> all = dictRepository.findAll(condition(dictQueryParam), PageParam.toPageable(param));
        List<Dict> data = all
            .get()
            .map(car -> {
                Dict vo = new Dict();
                BeanUtils.copyProperties(car, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<Dict> vo = PageVo.build(data, all.getTotalElements());
        return Result.ok(vo);
    }

    @Override
    public Result<String> selectDictLabel(String dictType, String dictValue) {
        return Result.ok(dictRepository.findDictLabel(dictType, dictValue));
    }

    @Override
    public Result<ArrayList<Dict>> selectDict(String dictType) {
        List<Dict> dictCache1 = DictUtils.getDictCache(dictType);
        ArrayList<Dict> dictCache = new ArrayList<>();
        if (dictCache1 != null) {
            dictCache = new ArrayList<>(dictCache1);
        }
        return Result.ok(dictCache);
    }

    @Override
    public Result<Dict> getDictInfo(Long dictCode) {
        return Result.ok(dictRepository.getById(dictCode));
    }

    @Override
    public Result<String> deleteDictById(IdParam idParam) {
        Optional<Dict> optional = dictRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Dict entity = optional.get();
        dictRepository.delete(entity);
        //刷新缓存
        List<Dict> byDictType = dictRepository.findByDictType(entity.getDictType());
        DictUtils.setDictCache(entity.getDictType(), byDictType);
        return Result.ok();
    }

    @Override
    public Result<String> saveDictInfo(DictParam dictParam) {
        if (has(dictParam.getDictLabel())) {
            return Result.no(Status.BAD_REQUEST, "标签已存在");
        }
        if (hasValue(dictParam.getDictType(), dictParam.getDictValue())) {
            return Result.no(Status.BAD_REQUEST, "该类型,字典值已存在");
        }
        Dict dict = new Dict();
        dict.setCreateTime(LocalDateTime.now());
        BeanUtils.copyProperties(dictParam, dict);
        dictRepository.saveAndFlush(dict);
        //刷新缓存
        List<Dict> byDictType = dictRepository.findByDictType(dict.getDictType());
        DictUtils.setDictCache(dict.getDictType(), byDictType);
        return Result.ok();
    }

    @Override
    public Result<String> updateDictInfo(DictParam dictParam, IdParam idParam) {
        Optional<Dict> optional = dictRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Dict entity = optional.get();
        if (!entity.getDictLabel().equals(dictParam.getDictLabel())) {
            if (has(dictParam.getDictLabel())) {
                //当修改了字典标签时
                return Result.no(Status.BAD_REQUEST, "标签已存在");
            }
        }
        if (
            !entity.getDictValue().equals(dictParam.getDictValue()) ||
            !entity.getDictType().equals(dictParam.getDictType())
        ) {
            //当修改了字典类型或字典值时
            if (hasValue(dictParam.getDictType(), dictParam.getDictValue())) {
                return Result.no(Status.BAD_REQUEST, "该类型,字典值已存在");
            }
        }
        BeanUtils.copyProperties(dictParam, entity);
        Dict dict = dictRepository.saveAndFlush(entity);
        List<Dict> byDictType = dictRepository.findByDictType(dict.getDictType());
        DictUtils.setDictCache(dict.getDictType(), byDictType);
        return Result.ok();
    }

    public Specification<Dict> condition(DictQueryParam dictQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (StrUtil.isNotBlank(dictQueryParam.getDictType())) {
                list.add(
                    criteriaBuilder.like(
                        root.get("dictType").as(String.class),
                        "%" + dictQueryParam.getDictType() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(dictQueryParam.getDictLabel())) {
                list.add(
                    criteriaBuilder.like(
                        root.get("dictLabel").as(String.class),
                        "%" + dictQueryParam.getDictLabel() + "%"
                    )
                );
            }
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }

    @Override
    public boolean has(String param) {
        Dict entity = Dict.builder().dictLabel(param).build();
        long count = dictRepository.count(Example.of(entity));
        return count > 0;
    }

    @Override
    public boolean hasValue(String dictType, String dictValue) {
        Dict entity = Dict.builder().dictType(dictType).dictValue(dictValue).build();
        long count = dictRepository.count(Example.of(entity));
        return count > 0;
    }
}
