/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.key.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.key.KeyAlarmRules;
import com.yiring.app.domain.key.KeyAlarmRulesRepository;
import com.yiring.app.excel.key.KeyAlarmRulesExportExcel;
import com.yiring.app.param.key.KeyAlarmRulesAddParam;
import com.yiring.app.param.key.KeyAlarmRulesExportParam;
import com.yiring.app.param.key.KeyAlarmRulesModifyParam;
import com.yiring.app.param.key.KeyAlarmRulesPageParam;
import com.yiring.app.service.key.KeyAlarmRulesService;
import com.yiring.app.vo.key.KeyAlarmRulesVo;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 按键报警规则
 *
 * @author LJ-2204
 * @date 2022/5/6
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class KeyAlarmRulesServiceImpl implements KeyAlarmRulesService {

    @Resource
    KeyAlarmRulesRepository keyAlarmRulesRepository;

    @Override
    public Result<String> add(KeyAlarmRulesAddParam keyAlarmRulesAddParam) {
        HashSet<User> users = new HashSet<>();
        String[] strings = keyAlarmRulesAddParam.getUserIds().split(",");
        Long[] ids = Convert.toLongArray(strings);
        if (ids.length < 3) {
            return Result.no(Status.BAD_REQUEST, "报警接收人至少3个");
        }
        for (Long id : ids) {
            users.add(User.builder().id(id).build());
        }
        KeyAlarmRules keyAlarmRules = KeyAlarmRules
            .builder()
            .department(Department.builder().id(keyAlarmRulesAddParam.getDeptId()).build())
            .users(users)
            .types(keyAlarmRulesAddParam.getTypes())
            .build();
        keyAlarmRulesRepository.save(keyAlarmRules);
        return Result.ok();
    }

    @Override
    public Result<String> delete(IdParam idParam) {
        Optional<KeyAlarmRules> keyAlarmRulesOptional = keyAlarmRulesRepository.findById(idParam.getId());

        if (keyAlarmRulesOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "报警规则不存在");

        keyAlarmRulesRepository.delete(keyAlarmRulesOptional.get());
        return Result.ok();
    }

    @Override
    public Result<String> modify(KeyAlarmRulesModifyParam keyAlarmRulesModifyParam) {
        Optional<KeyAlarmRules> keyAlarmRulesOptional = keyAlarmRulesRepository.findById(
            keyAlarmRulesModifyParam.getId()
        );

        if (keyAlarmRulesOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "报警规则不存在");

        HashSet<User> users = new HashSet<>();
        String[] strings = keyAlarmRulesModifyParam.getUserIds().split(",");
        Long[] ids = Convert.toLongArray(strings);
        if (ids.length < 3) {
            return Result.no(Status.BAD_REQUEST, "报警接收人至少3个");
        }
        for (Long id : ids) {
            users.add(User.builder().id(id).build());
        }

        KeyAlarmRules keyAlarmRules = keyAlarmRulesOptional.get();
        keyAlarmRules.setDepartment(Department.builder().id(keyAlarmRulesModifyParam.getDeptId()).build());
        keyAlarmRules.setUsers(users);
        keyAlarmRules.setTypes(keyAlarmRulesModifyParam.getTypes());

        keyAlarmRulesRepository.save(keyAlarmRules);
        return Result.ok();
    }

    @Override
    public void export(KeyAlarmRulesExportParam keyAlarmRulesExportParam, HttpServletResponse response) {
        Specification<KeyAlarmRules> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(keyAlarmRulesExportParam.getDeptIds())) {
                String[] ids = keyAlarmRulesExportParam.getDeptIds().split(",");
                CriteriaBuilder.In<Object> in = cb.in(
                    root.get(KeyAlarmRules.Fields.department).get(BasicEntity.Fields.id)
                );
                for (String id : ids) in.value(Convert.toLong(id));
                predicates.add(in);
            }

            if (StrUtil.isNotEmpty(keyAlarmRulesExportParam.getTypes())) {
                String[] types = keyAlarmRulesExportParam.getTypes().split(",");
                for (String type : types) {
                    predicates.add(cb.like(root.get(KeyAlarmRules.Fields.types), "%" + type + "%"));
                }
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        List<KeyAlarmRules> keyAlarmRules = keyAlarmRulesRepository.findAll(specification);
        List<KeyAlarmRulesExportExcel> keyAlarmRulesExportExcels = keyAlarmRules
            .stream()
            .map(KeyAlarmRulesExportExcel::transform)
            .collect(Collectors.toList());

        try (
            DefaultStreamExcelBuilder<KeyAlarmRulesExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(KeyAlarmRulesExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(keyAlarmRulesExportExcels);

            String fileName = URLEncoder.encode("按键报警规则信息.xlsx", StandardCharsets.UTF_8);

            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);

            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出按键报警规则信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<PageVo<KeyAlarmRulesVo>> page(KeyAlarmRulesPageParam keyAlarmRulesPageParam, PageParam pageParam) {
        Specification<KeyAlarmRules> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(keyAlarmRulesPageParam.getDeptIds())) {
                String[] ids = keyAlarmRulesPageParam.getDeptIds().split(",");
                CriteriaBuilder.In<Object> in = cb.in(
                    root.get(KeyAlarmRules.Fields.department).get(BasicEntity.Fields.id)
                );
                for (String id : ids) in.value(Convert.toLong(id));
                predicates.add(in);
            }

            if (StrUtil.isNotEmpty(keyAlarmRulesPageParam.getTypes())) {
                String[] types = keyAlarmRulesPageParam.getTypes().split(",");
                for (String type : types) {
                    predicates.add(cb.like(root.get(KeyAlarmRules.Fields.types), "%" + type + "%"));
                }
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<KeyAlarmRules> pages = keyAlarmRulesRepository.findAll(specification, pageable);
        List<KeyAlarmRulesVo> keyAlarmRulesVos = pages
            .getContent()
            .stream()
            .map(KeyAlarmRulesVo::transform)
            .collect(Collectors.toList());
        PageVo<KeyAlarmRulesVo> pageVo = PageVo.build(keyAlarmRulesVos, pages.getTotalElements());
        return Result.ok(pageVo);
    }
}
