/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.message.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yiring.app.domain.location.*;
import com.yiring.app.domain.log.ZyRealtimeLog;
import com.yiring.app.domain.log.ZyRealtimeLogRepository;
import com.yiring.app.service.message.ZyMessageService;
import com.yiring.app.util.GeoUtils;
import com.yiring.auth.domain.user.User;
import com.yiring.common.annotation.Times;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Arrays;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Point;
import org.springframework.data.domain.Example;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 真源消息处理实现类
 *
 * @author Jim
 * @version 0.1
 * 2022/5/9 10:22
 */

@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class ZyMessageServiceImpl implements ZyMessageService {

    @Resource
    LocationTagRepository locationTagRepository;

    @Resource
    LocationLogRepository locationLogRepository;

    @Resource
    SimpMessagingTemplate simpMessagingTemplate;

    @Resource
    ZyRealtimeLogRepository zyRealtimeLogRepository;

    @Times
    @Override
    public void consume(String message) {
        // 将消息转换成 JSON 格式
        JSONObject info = JSON.parseObject(message);
        log.info("Receiver Message: {}", info);

        // 解构消息内容
        JSONObject data = info.getJSONObject("params");
        String method = info.getString("method");

        // 记录日志
        ZyRealtimeLog realtimeLog = ZyRealtimeLog.builder().method(method).raw(info).build();
        zyRealtimeLogRepository.save(realtimeLog);

        // 业务处理
        switch (method) {
            // 实时定位
            case "position" -> processPositionMessage(data);
            // 设备低电量
            case "lowPower" -> processLowPowerMessage(data);
            // 设备状态变更
            case "deviceStatus" -> processDeviceStatusMessage(data);
            // 按键报警
            case "keyWarning" -> processKeyWarningMessage(data);
            // 围栏报警
            case "enclosure" -> log.warn("Ignore Message Type [enclosure]: {}", info);
            default -> log.warn("Unknown Message Type: {}", info);
        }
    }

    /**
     * 处理定位消息
     * @param data 消息内容
     */
    public void processPositionMessage(JSONObject data) {
        // TODO
        log.info("Position Message: {}", data);

        // 包装消息
        // TODO
        // 1. 解析消息内容，进行围栏、出入标识判断等处理，将定位记录录入数据库
        // 2. 创建一条需要进行消息推送的记录
        // 3. 将记录推送的消息推送模块
        // 4. 检查是否触发围栏告警，记录告警数据，并推送消息

        // 查询定位标签
        String tagId = data.getString("tagId");
        Example<LocationTag> example = Example.of(LocationTag.builder().code(tagId).build());
        Optional<LocationTag> optional = locationTagRepository.findOne(example);
        if (optional.isEmpty()) {
            log.warn("Tag Not Found: {}", tagId);
            return;
        }

        // 构建复合主键
        LocationTag tag = optional.get();
        TagTimeId id = new TagTimeId(tag);

        // 定位时间
        Instant instant = Instant.ofEpochMilli(data.getLongValue("locationTime"));
        LocalDateTime locationTime = LocalDateTime.ofInstant(instant, ZoneId.systemDefault());

        // 定位的基本信息
        LocationLog locationLog = LocationLog
            .builder()
            .id(id)
            .locationTime(locationTime)
            .areaId(data.getLong("areaId"))
            .floor(data.getString("floor"))
            .silent(data.getBoolean("silent"))
            .volt(data.getInteger("volt"))
            .voltUnit(data.getString("voltUnit"))
            .raw(data)
            .build();

        // 获取定位卡当前绑定的用户
        User user = tag.getUser();
        if (user != null) {
            // 定位标签当时所属的用户
            locationLog.setUser(user);
            // 查询当前用户的状态
            locationLog.setStatus(user.getStatus());
        }

        // 设置空间点位信息
        Point point = GeoUtils.createPoint(
            data.getDoubleValue("longitude"),
            data.getDoubleValue("latitude"),
            data.getDoubleValue("altitude")
        );
        locationLog.setPoint(point);

        // 定位信标
        Set<String> codes = Arrays
            .stream(data.getString("beacons").split(","))
            .map(beacon -> beacon.replaceAll("\\(.*\\)", ""))
            .collect(Collectors.toSet());
        locationLog.setBeacons(new JSONArray().fluentAddAll(codes));

        // TODO
        // 并计算出入标记（围栏、区域）

        // 写入数据
        locationLogRepository.saveAndFlush(locationLog);

        // 更新定位标签卡状态信息
        tag.setPoint(locationLog.getPoint());
        tag.setVolt(locationLog.getVolt());
        tag.setVoltUnit(locationLog.getVoltUnit());
        tag.setSilent(locationLog.getSilent());
        locationTagRepository.save(tag);

        // WebSocket 推送定位消息
        // 消息内容需要确定 TODO
        simpMessagingTemplate.convertAndSend("/topic/position", "{}");
        // TODO
        // 判断围栏告警是否触发，触发写入告警记录，并推送消息
    }

    /**
     * 处理低电量报警消息
     * @param data 消息内容
     */
    public void processLowPowerMessage(JSONObject data) {
        // TODO
        log.info("LowPower Message: {}", data);
    }

    /**
     * 处理设备状态更新消息
     * @param data 消息内容
     */
    public void processDeviceStatusMessage(JSONObject data) {
        // TODO
        log.info("DeviceStatus Message: {}", data);
    }

    /**
     * 处理按键报警消息
     * @param data 消息内容
     */
    public void processKeyWarningMessage(JSONObject data) {
        // TODO
        log.info("KeyWarning Message: {}", data);
    }
}
