/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.post.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.param.post.*;
import com.yiring.app.service.post.PostService;
import com.yiring.app.vo.post.PostIndexVo;
import com.yiring.app.vo.post.PostInfoVo;
import com.yiring.app.vo.post.PostVo;
import com.yiring.auth.domain.post.Post;
import com.yiring.auth.domain.post.PostRepository;
import com.yiring.auth.excel.post.PostExcel;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.IndexParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 职位信息
 * @author LJ-2204
 * @date 2022/4/13
 */
@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class PostServiceImpl implements PostService {

    @Resource
    PostRepository postRepository;

    @Override
    public Result<String> addPost(PostAddParam postAddParam) {
        // 判断职位是否存在
        if (hasPostInfo(postAddParam.getName())) return Result.no(Status.BAD_REQUEST, "职位已存在");

        Post post = PostAddParam.transform(postAddParam);
        postRepository.save(post);
        return Result.ok();
    }

    @Override
    public Result<String> modifyPost(PostModifyParam postModifyParam) {
        // 是否变更 职位名称
        Optional<Post> postOptional = postRepository.findOne(
            Example.of(Post.builder().id(postModifyParam.getId()).name(postModifyParam.getName()).build())
        );

        if (postOptional.isEmpty()) return Result.no(Status.BAD_REQUEST);

        Post post = postOptional.get();

        // 变更处理
        if (!StrUtil.equals(post.getName(), postModifyParam.getName()) && hasPostInfo(postModifyParam.getName())) {
            return Result.no(Status.BAD_REQUEST, "职位名称重复");
        }

        postModifyParam.transform(postModifyParam, post);
        postRepository.save(post);
        return Result.ok();
    }

    @Override
    public Result<String> deletePost(IdParam idParam) {
        Post post = Post.builder().id(idParam.getId()).build();
        postRepository.delete(post);
        return Result.ok();
    }

    @Override
    public Result<PageVo<PostVo>> findPostPage(PostFindParam postFindParam, PageParam pageParam) {
        Specification<Post> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(postFindParam.getName())) {
                predicates.add(cb.like(root.get(Post.Fields.name), "%" + postFindParam.getName() + "%"));
            }

            if (ObjectUtil.isNotEmpty(postFindParam.getEnable())) {
                predicates.add(cb.equal(root.get(Post.Fields.enable), postFindParam.getEnable()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        // 分页
        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<Post> page = postRepository.findAll(specification, pageable);
        List<PostVo> postVos = page.getContent().stream().map(PostVo::transform).collect(Collectors.toList());
        PageVo<PostVo> pageVo = PageVo.build(postVos, page.getTotalElements());
        return Result.ok(pageVo);
    }

    @Override
    public Result<PostInfoVo> findPostById(IdParam idParam) {
        Post post = postRepository.getById(idParam.getId());
        PostInfoVo postInfoVo = PostInfoVo.transform(post);
        return Result.ok(postInfoVo);
    }

    @Override
    public boolean hasPostInfo(String name) {
        Post post = Post.builder().name(name).build();
        return postRepository.count(Example.of(post)) > 0;
    }

    @Override
    public void exportPostInfo(PostExportParam postExportParam, HttpServletResponse response) {
        Specification<Post> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(postExportParam.getName())) {
                predicates.add(cb.like(root.get(Post.Fields.name), "%" + postExportParam.getName() + "%"));
            }

            if (ObjectUtil.isNotEmpty(postExportParam.getEnable())) {
                predicates.add(cb.equal(root.get(Post.Fields.enable), postExportParam.getEnable()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        List<Post> postList = postRepository.findAll(specification);
        List<PostExcel> postExcels = postList.stream().map(PostExcel::transform).collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<PostExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(PostExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(postExcels);

            String fileName = URLEncoder.encode("职位信息.xlsx", StandardCharsets.UTF_8);

            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);

            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出岗位信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<PageVo<PostIndexVo>> indexPost(IndexParam indexParam) {
        List<Post> posts = postRepository.findAll((root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(indexParam.getStr())) {
                predicates.add(cb.like(root.get(Post.Fields.name), "%" + indexParam.getStr() + "%"));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        });

        List<PostIndexVo> postIndexVos = posts.stream().map(PostIndexVo::transform).collect(Collectors.toList());
        PageVo<PostIndexVo> pageVo = PageVo.build(postIndexVos, postIndexVos.size());
        return Result.ok(pageVo);
    }
}
