/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.visitor.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.visitor.Car;
import com.yiring.app.domain.visitor.CarRepository;
import com.yiring.app.excel.visitor.CarExportExcel;
import com.yiring.app.param.visitor.CarQueryParam;
import com.yiring.app.service.visitor.CarService;
import com.yiring.app.vo.visitor.CarVo;
import com.yiring.auth.domain.role.Role;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * 2022/4/11 13:58
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class CarServiceImpl implements CarService {

    @Resource
    CarRepository carRepository;

    @Override
    public Result<Long> saveCarInfo(Car car) {
        // 检查车牌号车辆是否已经在园内
        if (carRepository.hasCarInfoByCarNum(car.getCarNum()) > 0) {
            return Result.no(Status.BAD_REQUEST, "您的车牌号已存在");
        }
        //        Car car = Car
        //            .builder()
        //            .carNum(carParam.getCarNum())
        //            .carType(carParam.getCarType())
        //            .labelCard(carParam.getLabelCard())
        //            .createTime(LocalDateTime.now())
        //            //            .visitorId(carParam.getVisitorId())
        //            .build();
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> updateCarInfo(Car car) {
        Optional<Car> optional = carRepository.findById(car.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car carOptional = optional.get();
        if (!car.getCarNum().equals(carOptional.getCarNum())) {
            // 当修改车牌号时才检查重复
            if (carRepository.hasCarInfoByCarNum(carOptional.getCarNum()) > 0) {
                return Result.no(Status.BAD_REQUEST, "您的车牌号已在园区内");
            }
        }
        BeanUtils.copyProperties(car, car);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardRec(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        car.setCardRecTime(LocalDateTime.now());
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardSend(IdParam idParam, String labelCard) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        car.setCardSendTime(LocalDateTime.now());
        car.setLabelCard(labelCard);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<String> deleteCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car entity = optional.get();
        carRepository.delete(entity);
        return Result.ok();
    }

    @Override
    public Result<CarVo> getCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Car entity = optional.get();
        CarVo vo = new CarVo();
        BeanUtils.copyProperties(entity, vo, Role.Fields.permissions);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<CarVo>> pageCarInfo(CarQueryParam carParam) {
        PageVo<CarVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Car.Fields.createTime));
        //如果传分页参数则分页,否查全部数据
        if (Objects.nonNull(carParam.getPageNo()) && Objects.nonNull(carParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(carParam.getPageNo() - 1, carParam.getPageSize());
            Page<Car> all = carRepository.findAll(condition(carParam), pageable);
            List<CarVo> data = all
                .get()
                .map(car -> {
                    CarVo vo = new CarVo();
                    BeanUtils.copyProperties(car, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Car> all = carRepository.findAll(condition(carParam), sort);
            List<CarVo> data = all
                .stream()
                .map(car -> {
                    CarVo vo = new CarVo();
                    BeanUtils.copyProperties(car, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public void export(CarQueryParam carParam, HttpServletResponse response) {
        Specification<Car> specification = condition(carParam);
        List<Car> all = carRepository.findAll(specification);
        List<CarExportExcel> data = all
            .stream()
            .map(car -> {
                CarExportExcel carExportExcel = new CarExportExcel();
                BeanUtils.copyProperties(car, carExportExcel);
                return carExportExcel;
            })
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<CarExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(CarExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(data);

            String fileName = URLEncoder.encode("车辆来访信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出车辆来访信息失败: " + e.getMessage());
        }
    }

    @Override
    public Specification<Car> condition(CarQueryParam carParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (StrUtil.isNotBlank(carParam.getCarNum())) {
                //车牌号码查询
                list.add(criteriaBuilder.like(root.get(Car.Fields.carNum), "%" + carParam.getCarNum() + "%"));
            }

            if (StrUtil.isNotBlank(carParam.getCarType())) {
                //车辆类型查询
                list.add(criteriaBuilder.like(root.get("carType").as(String.class), "%" + carParam.getCarType() + "%"));
            }

            if (StrUtil.isNotBlank(carParam.getDriverName())) {
                //司机名称查询
                list.add(
                    criteriaBuilder.like(root.get("driverName").as(String.class), "%" + carParam.getDriverName() + "%")
                );
            }

            if (StrUtil.isNotBlank(carParam.getDriverMobile())) {
                //司机电话号码查询
                list.add(
                    criteriaBuilder.like(
                        root.get("driverMobile").as(String.class),
                        "%" + carParam.getDriverMobile() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(carParam.getOrgId())) {
                //所属单位查询
                list.add(criteriaBuilder.like(root.get("orgId").as(String.class), "%" + carParam.getOrgId() + "%"));
            }

            if (StrUtil.isNotBlank(carParam.getReason())) {
                //来访原因查询
                list.add(criteriaBuilder.like(root.get("reason").as(String.class), "%" + carParam.getReason() + "%"));
            }

            if (Objects.nonNull(carParam.getCreateTimeStart()) && Objects.nonNull(carParam.getCreateTimeEnd())) {
                //来访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Car.Fields.cardSendTime),
                        carParam.getCreateTimeStart(),
                        carParam.getCreateTimeEnd()
                    )
                );
            }

            if (StrUtil.isNotBlank(carParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(root.get("labelCard").as(String.class), "%" + carParam.getLabelCard() + "%")
                );
            }

            //            if (StrUtil.isNotBlank(carParam.getLabelCardStatus())) {
            //                //标签卡状态查询
            //                list.add(
            //                    criteriaBuilder.equal(root.get("labelCardStatus").as(String.class), carParam.getLabelCardStatus())
            //                );
            //            }

            if (Objects.nonNull(carParam.getCardRecTimeStart()) && Objects.nonNull(carParam.getCardRecTimeEnd())) {
                //收卡时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Car.Fields.cardRecTime),
                        carParam.getCardRecTimeStart(),
                        carParam.getCardRecTimeEnd()
                    )
                );
            }

            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
