/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.visitor.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.visitor.Car;
import com.yiring.app.domain.visitor.Visitor;
import com.yiring.app.domain.visitor.VisitorRepository;
import com.yiring.app.excel.visitor.VisitorExportExcel;
import com.yiring.app.param.visitor.VisitorAppQueryParam;
import com.yiring.app.param.visitor.VisitorParam;
import com.yiring.app.param.visitor.VisitorQueryParam;
import com.yiring.app.service.visitor.CarService;
import com.yiring.app.service.visitor.VisitorService;
import com.yiring.app.vo.visitor.VisitorAppVo;
import com.yiring.app.vo.visitor.VisitorDetailsVo;
import com.yiring.app.vo.visitor.VisitorVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.constant.RegEx;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * 2022/4/19 14:12
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class VisitorServiceImpl implements VisitorService {

    @Resource
    VisitorRepository visitorRepository;

    @Resource
    CarService carService;

    @Override
    public Result<String> saveVisitor(VisitorParam visitorParam) {
        //设置创建时间和默认状态
        Visitor visitor = Visitor
            .builder()
            .createTime(LocalDateTime.now())
            .updateTime(LocalDateTime.now())
            .cardSendTime(LocalDateTime.now())
            .build();
        BeanUtils.copyProperties(visitorParam, visitor);
        if (visitorParam.getMainVisitorId() != null) {
            //如果主访人员id不为空,
            Visitor optional = visitorRepository.findByIdAndDeleted(visitorParam.getMainVisitorId(), "N");
            if (optional == null) {
                return Result.no(Status.NOT_FOUND, "信息失效,请联系管理员");
            }
            visitor.setMainVisitorId(visitorParam.getMainVisitorId());
        }

        //如果有车, 填车辆信息,
        if (visitorParam.getHasCar() == 1) {
            //校验车辆必填信息
            if (StrUtil.isBlank(visitorParam.getCarNum())) {
                return Result.no(Status.BAD_REQUEST, "请输入车牌号码");
            }
            if (visitorParam.getCarNum().matches(RegEx.CARNUM)) {
                return Result.no(Status.BAD_REQUEST, "请输入正确的车牌号码");
            }
            if (StrUtil.isBlank(visitorParam.getCarType())) {
                return Result.no(Status.BAD_REQUEST, "请输入车辆类型");
            }
            if (StrUtil.isBlank(visitorParam.getLabelCard())) {
                return Result.no(Status.BAD_REQUEST, "请输入车辆标签卡");
            }
            //需要判断标签卡是否有效
            Car car = Car
                .builder()
                .carNum(visitorParam.getCarNum())
                .carType(visitorParam.getCarType())
                .labelCard(visitorParam.getLabelCard())
                .createTime(LocalDateTime.now())
                .build();
            carService.saveCarInfo(car);
            visitor.setCar(car);
        }
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<String> updateVisitor(VisitorParam visitorParam, IdParam idParam) {
        Visitor optional = visitorRepository.findByIdAndDeleted(idParam.getId(), "N");
        if (optional == null) {
            return Result.no(Status.NOT_FOUND);
        }
        //判断标签卡是否正确  生效
        Car car = optional.getCar();
        if (car != null) {
            BeanUtils.copyProperties(visitorParam, car);
            carService.updateCarInfo(car);
        } else {
            if (StrUtil.isNotBlank(visitorParam.getCarNum())) {
                car =
                    Car
                        .builder()
                        .carNum(visitorParam.getCarNum())
                        .carType(visitorParam.getCarType())
                        .labelCard(visitorParam.getLabelCard())
                        .createTime(LocalDateTime.now())
                        .build();
                optional.setCar(car);
                carService.saveCarInfo(car);
            } else {
                optional.setCar(null);
            }
        }
        BeanUtils.copyProperties(visitorParam, optional);
        optional.setUpdateTime(LocalDateTime.now());
        visitorRepository.saveAndFlush(optional);
        return Result.ok();
    }

    @Override
    public Result<VisitorDetailsVo> selectVisitor(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Visitor visitor = optional.get();
        List<Visitor> list = new ArrayList<>();
        List<VisitorDetailsVo.VisitorResultVo> listResult = new ArrayList<>();
        //主访人id为空 为主访问人,查询出其下面的随访人的姓名和电话
        if (visitor.getMainVisitorId() == null) {
            list = visitorRepository.findAllByMainVisitorIdAndDeletedEquals(visitor.getId(), "N");
        } else {
            //查询此随访访客的主访客,
            Visitor n = visitorRepository.findByIdAndDeleted(visitor.getMainVisitorId(), "N");
            if (n != null) {
                //添加此随访人一同的随访人
                list =
                    visitorRepository.findAllByMainVisitorIdAndDeletedEqualsAndIdIsNot(n.getId(), "N", visitor.getId());
                list.add(n);
            }
        }
        //处理返回数据的结构
        if (list != null) {
            list.forEach(visitorEntity -> {
                VisitorDetailsVo.VisitorResultVo visitorResultVo = new VisitorDetailsVo.VisitorResultVo();
                BeanUtils.copyProperties(visitorEntity, visitorResultVo);
                listResult.add(visitorResultVo);
            });
        }
        VisitorDetailsVo vo = new VisitorDetailsVo();
        BeanUtils.copyProperties(visitor, vo);
        VisitorDetailsVo.UserVo userVo = VisitorDetailsVo.UserVo
            .builder()
            .realName(visitor.getUser().getRealName())
            .mobile(visitor.getUser().getMobile())
            .deptName(visitor.getUser().getDepartment().getName())
            .build();
        vo.setUserVo(userVo);
        vo.setList(listResult);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<VisitorVo>> pageVisitor(VisitorQueryParam visitorQueryParam) {
        PageVo<VisitorVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Visitor.Fields.createTime));
        if (Objects.nonNull(visitorQueryParam.getPageNo()) && Objects.nonNull(visitorQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(visitorQueryParam.getPageNo() - 1, visitorQueryParam.getPageSize());
            Page<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), pageable);
            List<VisitorVo> data = all
                .get()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    VisitorVo.UserVo userVo = VisitorVo.UserVo
                        .builder()
                        .realName(visitor.getUser().getRealName())
                        .build();
                    vo.setUserVo(userVo);
                    //翻译被访问人姓名
                    //                    if (vo.getIntervieweeId() != null) {
                    //                        Optional<User> byId = userRepository.findById(vo.getIntervieweeId());
                    //                        byId.ifPresent(user -> vo.setIntervieweeName(user.getRealName()));
                    //                    }
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), sort);
            List<VisitorVo> data = all
                .stream()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    VisitorVo.UserVo userVo = VisitorVo.UserVo
                        .builder()
                        .realName(visitor.getUser().getRealName())
                        .build();
                    vo.setUserVo(userVo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public void exportVisitor(VisitorQueryParam visitorQueryParam, HttpServletResponse response) {
        List<Visitor> visitors = visitorRepository.findAll(condition(visitorQueryParam));
        List<VisitorExportExcel> visitorExportExcels = visitors
            .stream()
            .map(visitor -> {
                VisitorExportExcel visitorExportExcel = new VisitorExportExcel();
                BeanUtils.copyProperties(visitor, visitorExportExcel);
                return visitorExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<VisitorExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(VisitorExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(visitorExportExcels);

            String fileName = URLEncoder.encode("访客信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出访客信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> cardSend(String labelCard, IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        //状态变更为使用中
        visitor.setLabelCardStatus("2");
        visitor.setCardSendTime(LocalDateTime.now());
        visitor.setLabelCard(labelCard);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<String> cardRec(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        visitor.setLabelCardStatus("3");
        visitor.setUpdateTime(LocalDateTime.now());
        visitor.setCardRecTime(LocalDateTime.now());
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<PageVo<VisitorAppVo>> selectInfo(VisitorAppQueryParam visitorAppQueryParam) {
        PageVo<VisitorAppVo> resultVo;
        //排序

        //分页
        Pageable pageable = PageRequest.of(visitorAppQueryParam.getPageNo() - 1, visitorAppQueryParam.getPageSize());
        Page<Visitor> all = visitorRepository.findAllByVisitorMobileAndDeletedEquals(
            visitorAppQueryParam.getVisitorMobile(),
            "N",
            pageable
        );
        List<VisitorAppVo> data = all
            .get()
            .map(visitor -> {
                VisitorAppVo vo = new VisitorAppVo();
                vo.setId(visitor.getId());
                vo.setCardSendTime(visitor.getCardSendTime());
                VisitorAppVo.UserVo userVo = VisitorAppVo.UserVo
                    .builder()
                    .realName(visitor.getUser().getRealName())
                    .mobile(visitor.getVisitorMobile())
                    .build();
                vo.setUserVo(userVo);
                return vo;
            })
            .collect(Collectors.toList());
        resultVo = PageVo.build(data, all.getTotalElements());
        return Result.ok(resultVo);
    }

    public Specification<Visitor> condition(VisitorQueryParam visitorQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            Join<Visitor, User> join = root.join("user", JoinType.LEFT);
            //查询条件
            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorName())) {
                //访客姓名
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorName),
                        "%" + visitorQueryParam.getVisitorName() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorMobile())) {
                //访客电话
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorMobile),
                        "%" + visitorQueryParam.getVisitorMobile() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorCarNum())) {
                //访客身份证
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorCarNum),
                        "%" + visitorQueryParam.getVisitorCarNum() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getTaskType())) {
                //是否特殊作业
                list.add(
                    criteriaBuilder.like(root.get(Visitor.Fields.taskType), "%" + visitorQueryParam.getTaskType() + "%")
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getOrgId())) {
                //所属单位查询
                list.add(
                    criteriaBuilder.like(root.get(Visitor.Fields.orgId), "%" + visitorQueryParam.getOrgId() + "%")
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorSource())) {
                //访客来源
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorSource),
                        "%" + visitorQueryParam.getVisitorSource() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getIntervieweeId())) {
                //被访人
                list.add(criteriaBuilder.equal(join.get(BasicEntity.Fields.id), visitorQueryParam.getIntervieweeId()));
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.labelCard),
                        "%" + visitorQueryParam.getLabelCard() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getIsMain())) {
                //标签卡号查询
                if (visitorQueryParam.getIsMain().equals("1")) {
                    list.add(criteriaBuilder.isNull(root.get(Visitor.Fields.mainVisitorId)));
                } else {
                    list.add(criteriaBuilder.isNotNull(root.get(Visitor.Fields.mainVisitorId)));
                }
            }

            //            if (StrUtil.isNotBlank(visitorQueryParam.getLabelCardStatus())) {
            //                //标签卡状态
            //                list.add(
            //                    criteriaBuilder.like(
            //                        root.get(Visitor.Fields.labelCardStatus),
            //                        "%" + visitorQueryParam.getLabelCardStatus() + "%"
            //                    )
            //                );
            //            }

            if (
                Objects.nonNull(visitorQueryParam.getAppOfVisitTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getAppOfVisitTimeEnd())
            ) {
                //预约到访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.appOfVisitTime),
                        visitorQueryParam.getAppOfVisitTimeStart(),
                        visitorQueryParam.getAppOfVisitTimeEnd()
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardSendTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardSendTimeEnd())
            ) {
                //实际到访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardSendTime),
                        visitorQueryParam.getCardSendTimeStart(),
                        visitorQueryParam.getCardSendTimeEnd()
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardRecTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardRecTimeEnd())
            ) {
                //收卡时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardRecTime),
                        visitorQueryParam.getCardRecTimeStart(),
                        visitorQueryParam.getCardRecTimeEnd()
                    )
                );
            }
            Order order = criteriaBuilder.desc(root.get(Visitor.Fields.updateTime));
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            query.orderBy(order);
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
