/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.domain.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.post.Post;
import com.yiring.auth.domain.role.Role;
import com.yiring.common.annotation.FieldMapping;
import com.yiring.common.domain.BasicEntity;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.Hibernate;
import org.hibernate.annotations.Comment;

/**
 * 用户
 *
 * @author ifzm
 * 2018/9/3 15:27
 */

@Getter
@Setter
@ToString
@SuperBuilder(toBuilder = true)
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(
    name = "SYS_USER",
    indexes = {
        @Index(columnList = "mobile", unique = true),
        @Index(columnList = "username", unique = true),
        @Index(columnList = "email", unique = true),
        @Index(columnList = "enabled"),
        @Index(columnList = "deleted"),
    }
)
@Comment("系统用户")
public class User extends BasicEntity implements Serializable {

    @Serial
    private static final long serialVersionUID = -5787847701210907511L;

    /**
     * 数据来源于【真源人员定位系统 - 定位信标】
     * 作用: 用于双向联动进行数据同步
     */
    @FieldMapping("id")
    @Comment("外链主键")
    Long linkId;

    @Comment("厂区 ID")
    Long orgId;

    @Comment("工号（唯一标识）")
    @Column(unique = true)
    String uuid;

    @Comment("真实姓名")
    String realName;

    @Comment("用户名")
    @Column(unique = true)
    String username;

    @Comment("手机号")
    @Column(unique = true)
    String mobile;

    @Comment("邮箱")
    @Column(unique = true)
    String email;

    @Comment("密码")
    String password;

    @Comment("简介")
    String introduction;

    @Comment("头像")
    String avatar;

    @Comment("是否启用")
    Boolean enabled;

    @Comment("是否删除")
    Boolean deleted;

    @Comment("是否特殊")
    Boolean isSpecial;

    @Comment("性别")
    Boolean gender;

    @JsonIgnore
    @Builder.Default
    @Comment("角色集合")
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(
        name = "SYS_USER_ROLES",
        joinColumns = { @JoinColumn(name = "user_id") },
        inverseJoinColumns = { @JoinColumn(name = "role_id") }
    )
    Set<Role> roles = new HashSet<>();

    @Comment("所属部门")
    @ManyToOne
    @JsonIgnore
    @JoinColumn(name = "department_id")
    Department department;

    @Comment("所属职称")
    @ManyToOne
    @JoinColumn(name = "post_id")
    Post post;

    @Comment("用户类型")
    @Enumerated(EnumType.STRING)
    Type type;

    @Comment("单位（外部）")
    String unit;

    @Comment("状态")
    @Builder.Default
    @Enumerated(EnumType.STRING)
    Status status = Status.DEFAULT;

    @Comment("最后登录IP地址")
    String lastLoginIp;

    @Comment("最后登录时间")
    LocalDateTime lastLoginTime;

    @SuppressWarnings({ "unused" })
    public enum Type {
        EMPLOYEES("员工"),
        GUEST("访客");

        final String text;

        Type(String text) {
            this.text = text;
        }

        public String text() {
            return this.text;
        }
    }

    @SuppressWarnings({ "unused" })
    public enum Status {
        DEFAULT("默认"),
        REST("休息"),
        WORK("工作"),
        INSPECTION("巡检"),
        SPECIAL_JOB("特殊作业");

        final String text;

        Status(String text) {
            this.text = text;
        }

        public String text() {
            return this.text;
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;
        User user = (User) o;
        return getId() != null && Objects.equals(getId(), user.getId());
    }

    @Override
    public int hashCode() {
        return getClass().hashCode();
    }
}
