/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.permission;

import com.yiring.auth.domain.permission.Permission;
import com.yiring.auth.domain.permission.PermissionRepository;
import com.yiring.auth.param.permission.PermissionParam;
import com.yiring.auth.util.Permissions;
import com.yiring.auth.vo.permission.PermissionVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.Resource;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 系统权限管理控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/3/25 15:27
 */

@Slf4j
@Validated
@Api(tags = "Permission")
@RestController
@RequestMapping("/manage/permission/")
public class PermissionController {

    @Resource
    PermissionRepository permissionRepository;

    @ApiOperation(value = "新增")
    @PostMapping("add")
    public Result<String> add(@Valid PermissionParam param) {
        if (has(param.getUid())) {
            return Result.no(Status.BAD_REQUEST, "权限标识重复");
        }

        Permission entity = new Permission();
        BeanUtils.copyProperties(param, entity);
        entity.setTree(getTreeNode(param.getPid()));
        permissionRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "修改")
    @PostMapping("modify")
    public Result<String> modify(@Valid PermissionParam param, @Valid IdParam idParam) {
        Optional<Permission> optional = permissionRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Permission entity = optional.get();
        if (!entity.getUid().equals(param.getUid())) {
            // 仅当修改了角色标识时才检查重复
            if (has(param.getUid())) {
                return Result.no(Status.BAD_REQUEST, "权限标识重复");
            }
        }

        BeanUtils.copyProperties(param, entity);
        entity.setTree(getTreeNode(param.getPid()));
        permissionRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "删除")
    @PostMapping("deleted")
    public Result<String> deleted(@Valid IdParam param) {
        Optional<Permission> optional = permissionRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Permission entity = optional.get();
        permissionRepository.delete(entity);
        return Result.ok();
    }

    @ApiOperation(value = "查询")
    @GetMapping("find")
    public Result<PermissionVo> find(@Valid IdParam param) {
        Optional<Permission> optional = permissionRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        PermissionVo vo = new PermissionVo();
        BeanUtils.copyProperties(optional.get(), vo);
        return Result.ok(vo);
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("page")
    public Result<PageVo<PermissionVo>> page(@Valid PageParam param) {
        Page<Permission> page = permissionRepository.findAll(PageParam.toPageable(param));
        List<PermissionVo> data = Permissions.toPermissionVos(page.toList());
        PageVo<PermissionVo> vo = PageVo.build(data, page.getTotalElements());
        return Result.ok(vo);
    }

    @ApiOperation(value = "树结构查询")
    @GetMapping(value = "tree", headers = "Content-Type=" + MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public Result<ArrayList<PermissionVo>> tree(
        @ApiParam(value = "父级 id", example = "0") @RequestParam(required = false) Long pid
    ) {
        List<Permission> permissions = permissionRepository.findAll();
        List<PermissionVo> vos = Permissions.toTree(permissions, Objects.isNull(pid) ? 0L : pid);
        return Result.ok((ArrayList<PermissionVo>) vos);
    }

    /**
     * 根据父级 ID 获取当前树节点标识
     * @param pid 父级 ID
     * @return 树节点标识
     */
    private String getTreeNode(Long pid) {
        Optional<Permission> parent = permissionRepository.findById(pid);
        if (parent.isEmpty()) {
            return String.valueOf(pid);
        } else {
            return String.format("%s.%s", parent.get().getTree(), pid);
        }
    }

    /**
     * 检查是否存在已有相同标识的权限
     * @param uid 权限标识
     * @return 是否存在
     */
    private boolean has(String uid) {
        Permission entity = Permission.builder().uid(uid).build();
        return permissionRepository.count(Example.of(entity)) > 0;
    }
}
