/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.role;

import com.yiring.auth.domain.permission.Permission;
import com.yiring.auth.domain.permission.PermissionRepository;
import com.yiring.auth.domain.role.Role;
import com.yiring.auth.domain.role.RoleRepository;
import com.yiring.auth.param.IdsParam;
import com.yiring.auth.param.role.RoleParam;
import com.yiring.auth.util.Permissions;
import com.yiring.auth.vo.role.RoleVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 系统角色管理控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/3/25 15:27
 */

@Slf4j
@Validated
@Api(tags = "Role")
@RestController
@RequestMapping("/manage/role/")
public class RoleController {

    @Resource
    RoleRepository roleRepository;

    @Resource
    PermissionRepository permissionRepository;

    @ApiOperation(value = "新增")
    @PostMapping("add")
    public Result<String> add(@Valid RoleParam param) {
        if (has(param.getUid())) {
            return Result.no(Status.BAD_REQUEST, "角色标识重复");
        }

        Role entity = new Role();
        BeanUtils.copyProperties(param, entity);
        roleRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "修改")
    @PostMapping("modify")
    public Result<String> modify(@Valid RoleParam param, @Valid IdParam idParam) {
        Optional<Role> optional = roleRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Role entity = optional.get();
        if (!entity.getUid().equals(param.getUid())) {
            // 仅当修改了角色标识时才检查重复
            if (has(param.getUid())) {
                return Result.no(Status.BAD_REQUEST, "角色标识重复");
            }
        }

        BeanUtils.copyProperties(param, entity);
        roleRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "分配权限")
    @PostMapping("assign")
    public Result<String> assign(@Valid IdParam idParam, @Valid IdsParam idsParam) {
        Optional<Role> optional = roleRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        // 查询权限集合
        Set<Long> ids = idsParam.toIds();
        Set<Permission> permissions = permissionRepository
            .findAll()
            .stream()
            .filter(permission -> ids.contains(permission.getId()))
            .collect(Collectors.toSet());

        Role entity = optional.get();
        entity.setPermissions(permissions);
        roleRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "删除")
    @PostMapping("deleted")
    public Result<String> deleted(@Valid IdParam param) {
        Optional<Role> optional = roleRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Role entity = optional.get();
        roleRepository.delete(entity);
        return Result.ok();
    }

    @ApiOperation(value = "查询")
    @GetMapping("find")
    public Result<RoleVo> find(@Valid IdParam param) {
        Optional<Role> optional = roleRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Role entity = optional.get();
        RoleVo vo = new RoleVo();
        BeanUtils.copyProperties(entity, vo, Role.Fields.permissions);
        vo.setPermissions(Permissions.toPermissionVos(Permissions.toPermissions(Collections.singleton(entity))));
        return Result.ok(vo);
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("page")
    public Result<PageVo<RoleVo>> page(@Valid PageParam param) {
        Page<Role> page = roleRepository.findAll(PageParam.toPageable(param));
        List<RoleVo> data = new ArrayList<>(Permissions.toRoleVos(page.toSet()));
        PageVo<RoleVo> vo = PageVo.build(data, page.getTotalElements());
        return Result.ok(vo);
    }

    @ApiOperation(value = "选项查询")
    @GetMapping("selector")
    public Result<ArrayList<RoleVo>> selector() {
        List<Role> roles = roleRepository.findAll();
        List<RoleVo> vos = Permissions.toRoleVos(new HashSet<>(roles));
        return Result.ok((ArrayList<RoleVo>) vos);
    }

    /**
     * 检查是否存在已有相同标识的角色
     * @param uid 角色标识
     * @return 是否存在
     */
    private boolean has(String uid) {
        Role entity = Role.builder().uid(uid).build();
        return roleRepository.count(Example.of(entity)) > 0;
    }
}
