/* (C) 2021 YiRing, Inc. */
package com.yiring.common.core;

import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import lombok.AccessLevel;
import lombok.Builder;
import lombok.Data;
import lombok.experimental.FieldDefaults;
import lombok.extern.slf4j.Slf4j;

/**
 * 标准的响应对象（所有的接口响应内容格式都应该是一致的）
 *
 * @author ifzm
 * @version 1.1
 * 2018/9/4 11:05
 */

@SuppressWarnings({ "unchecked", "unused" })
@ApiModel("Result")
@JsonInclude(JsonInclude.Include.NON_NULL)
@Slf4j
@Data
@Builder
@FieldDefaults(level = AccessLevel.PRIVATE)
public class Result<T extends Serializable> implements Serializable {

    @Serial
    private static final long serialVersionUID = -4802543396830024571L;

    /**
     * 接口响应时间
     */
    @ApiModelProperty(value = "响应时间", example = "2021-01-01 00:00:00")
    String timestamp;

    /**
     * 接口耗时（单位：秒）通常在调试阶段出现
     */
    @ApiModelProperty(value = "耗时", example = "0.001s")
    String times;

    /**
     * 响应状态码
     */
    @ApiModelProperty(value = "状态码", example = "200")
    Integer status;

    /**
     * 业务标识码
     */
    @ApiModelProperty(value = "业务标识码", example = "0")
    Integer code;

    /**
     * 响应消息
     */
    @ApiModelProperty(value = "消息", example = "OK")
    String message;

    /**
     * 详细信息，通常为参数校验结果或自定义消息
     */
    @ApiModelProperty(value = "详细信息", example = "Details message")
    String details;

    /**
     * 异常信息，通常在出现服务器错误时会出现该异常
     */
    @ApiModelProperty(value = "异常信息", notes = "出现错误时会出现该字段", example = "Error message")
    String error;

    /**
     * 响应内容
     */
    @ApiModelProperty("内容")
    T body;

    /**
     * 返回成功响应内容（默认）
     *
     * @return Result
     * @see com.yiring.common.core.Status
     */
    public static <T extends Serializable> Result<T> ok() {
        return (Result<T>) Result.builder().status(Status.OK.value()).message(Status.OK.getReasonPhrase()).build();
    }

    /**
     * 返回自定义body的成功响应内容
     *
     * @param body {@link Object}
     * @return Result
     */
    public static <T extends Serializable> Result<T> ok(T body) {
        return (Result<T>) Result
            .builder()
            .status(Status.OK.value())
            .message(Status.OK.getReasonPhrase())
            .body(body)
            .build();
    }

    /**
     * 返回失败响应内容
     *
     * @return Result
     * @see Status#BAD_REQUEST
     */
    public static <T extends Serializable> Result<T> no(Status status) {
        return no(status, null, null, null);
    }

    /**
     * 返回失败响应内容
     *
     * @return Result
     * @see Status
     */
    public static <T extends Serializable> Result<T> no(Status status, String details) {
        return no(status, null, details, null);
    }

    /**
     * 返回失败响应内容
     *
     * @return Result
     * @see Status
     */
    public static <T extends Serializable> Result<T> no(Status status, Throwable error) {
        return no(status, null, null, error);
    }

    /**
     * 返回失败响应内容
     *
     * @return Result
     * @see Status
     */
    public static <T extends Serializable> Result<T> no(Status status, Integer code, String details, Throwable error) {
        Result<T> result = (Result<T>) Result
            .builder()
            .status(status.value())
            .message(status.getReasonPhrase())
            .code(code)
            .details(details)
            .build();

        if (error != null) {
            result.setError(error.getMessage());
        }

        return result;
    }
}
