/* (C) 2021 YiRing, Inc. */
package com.yiring.common.core;

import org.springframework.lang.Nullable;

/**
 * API 响应状态码
 * 包含系统和业务两个维度
 */

@SuppressWarnings({ "unused" })
public enum Status {
    /**
     * 成功
     */
    OK(200, "OK"),

    /**
     * 用户认证失败
     */
    NON_AUTHORITATIVE_INFORMATION(203, "认证失败"),

    /**
     * 失败的请求，通常是一些验证错误
     */
    BAD_REQUEST(400, "FAIL"),

    /**
     * 鉴权失败
     */
    UNAUTHORIZED(401, "凭证过期"),

    /**
     * Token 错误/失效
     */
    FORBIDDEN(403, "禁止访问"),

    /**
     * 找不到资源
     */
    NOT_FOUND(404, "Not Found"),

    /**
     * 不支持的请求类型
     */
    METHOD_NOT_ALLOWED(405, "不支持的请求类型"),

    /**
     * 参数校验失败
     */
    EXPECTATION_FAILED(417, "无效参数"),

    /**
     * 服务器错误
     */
    INTERNAL_SERVER_ERROR(500, "服务器错误"),

    /**
     * 未知错误
     */
    UNKNOWN_ERROR(500, "未知错误"),

    /**
     * API 未实现
     */
    NOT_IMPLEMENTED(501, "API 未实现"),

    /**
     * 服务异常（网关提醒）
     */
    BAD_GATEWAY(502, "Bad Gateway"),

    /**
     * 服务暂停（网关提醒）
     */
    SERVICE_UNAVAILABLE(503, "Service Unavailable");

    private final int value;

    private final String reasonPhrase;

    Status(int value, String reasonPhrase) {
        this.value = value;
        this.reasonPhrase = reasonPhrase;
    }

    /**
     * Return the enum constant of this type with the specified numeric value.
     *
     * @param statusCode the numeric value of the enum to be returned
     * @return the enum constant with the specified numeric value
     * @throws IllegalArgumentException if this enum has no constant for the specified numeric value
     */
    public static Status valueOf(int statusCode) {
        Status status = resolve(statusCode);
        if (status == null) {
            throw new IllegalArgumentException("No matching constant for [" + statusCode + "]");
        }
        return status;
    }

    /**
     * Resolve the given status code to an {@code ResultStatus}, if possible.
     *
     * @param statusCode the HTTP status code (potentially non=standard)
     * @return the corresponding {@code ResultStatus}, or {@code null} if not found
     * @since 5.0
     */
    @Nullable
    public static Status resolve(int statusCode) {
        for (Status status : values()) {
            if (status.value == statusCode) {
                return status;
            }
        }
        return null;
    }

    /**
     * Return the integer value of this status code.
     */
    public int value() {
        return this.value;
    }

    /**
     * Return the reason phrase of this status code.
     */
    public String getReasonPhrase() {
        return this.reasonPhrase;
    }

    /**
     * Return a string representation of this status code.
     */
    @Override
    public String toString() {
        return this.value + " " + name();
    }
}
