/* (C) 2021 YiRing, Inc. */
package com.yiring.common.aspect;

import cn.hutool.extra.servlet.ServletUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.yiring.common.constant.DateFormatter;
import com.yiring.common.core.Result;
import com.yiring.common.util.Commons;
import java.time.LocalDateTime;
import javax.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

/**
 * 请求接口切面，记录接口耗时相关
 *
 * @author ifzm
 * @version 0.1
 */

@Slf4j
@Aspect
@Component
public class RequestAspect {

    @Value("${debug}")
    Boolean debug;

    @Pointcut(
        "@annotation(org.springframework.web.bind.annotation.PostMapping) || @annotation(org.springframework.web.bind.annotation.GetMapping) || @annotation(org.springframework.web.bind.annotation.PutMapping) || @annotation(org.springframework.web.bind.annotation.DeleteMapping) || @annotation(org.springframework.web.bind.annotation.PatchMapping) || @annotation(org.springframework.web.bind.annotation.ExceptionHandler)"
    )
    public void apiPointCut() {}

    @Around("apiPointCut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        HttpServletRequest request = getRequest();

        long start = System.currentTimeMillis();
        Object result = point.proceed();
        long end = System.currentTimeMillis();

        String timestamp = LocalDateTime.now().format(DateFormatter.DATE_TIME);
        String times = String.format("%.3fs", (double) (end - start) / 1000);

        // Print Request Log (Optional Replace: MDC)
        String extra = "";
        if (Boolean.TRUE.equals(debug)) {
            String headers = JSONObject.toJSONString(ServletUtil.getHeaderMap(request), SerializerFeature.PrettyFormat);
            String params = JSONObject.toJSONString(ServletUtil.getParamMap(request), SerializerFeature.PrettyFormat);
            extra += String.format("\nHeaders: %s", headers);
            extra += String.format("\nParams: %s", params);
            if (result instanceof Result) {
                extra +=
                    String.format(
                        "\nResponse: %s, %s",
                        ((Result<?>) result).getStatus(),
                        ((Result<?>) result).getMessage()
                    );
            }
        }
        log.info(
            "[Request] URL: {}, IP: {}, Times: {}{}",
            request.getRequestURL(),
            Commons.getClientIpAddress(request),
            times,
            extra
        );

        if (result instanceof Result) {
            ((Result<?>) result).setTimestamp(timestamp);
            ((Result<?>) result).setTimes(times);
        }

        return result;
    }

    private HttpServletRequest getRequest() {
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        assert requestAttributes != null;
        return (HttpServletRequest) requestAttributes.resolveReference(RequestAttributes.REFERENCE_REQUEST);
    }
}
