/* (C) 2022 YiRing, Inc. */
package com.yiring.common.util;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.yiring.common.core.Redis;
import com.yiring.common.dict.Dict;
import java.util.Collection;
import java.util.List;

/**
 * 字典工具类
 *
 * @author tzl
 */
public class DictUtils {

    /**
     * 分隔符
     */
    public static final String SEPARATOR = ",";

    /**
     * 设置字典缓存
     *
     * @param key       参数键
     * @param dictDatas 字典数据列表
     */
    public static void setDictCache(String key, List<Dict> dictDatas) {
        SpringUtil.getBean(Redis.class).set(getCacheKey(key), dictDatas);
    }

    /**
     * 获取字典缓存
     *
     * @param key 参数键
     * @return dictDatas 字典数据列表
     */
    public static List<Dict> getDictCache(String key) {
        Object cacheObj = SpringUtil.getBean(Redis.class).get(getCacheKey(key));
        if (ObjectUtil.isNotNull(cacheObj)) {
            return cast(cacheObj);
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    public static <T> T cast(Object obj) {
        return (T) obj;
    }

    /**
     * 根据字典类型和字典值获取字典标签
     *
     * @param dictType  字典类型
     * @param dictValue 字典值
     * @return 字典标签
     */
    public static String getDictLabel(String dictType, String dictValue) {
        return getDictLabel(dictType, dictValue, SEPARATOR);
    }

    /**
     * 根据字典类型和字典标签获取字典值
     *
     * @param dictType  字典类型
     * @param dictLabel 字典标签
     * @return 字典值
     */
    public static String getDictValue(String dictType, String dictLabel) {
        return getDictValue(dictType, dictLabel, SEPARATOR);
    }

    /**
     * 根据字典类型和字典值获取字典标签
     *
     * @param dictType  字典类型
     * @param dictValue 字典值
     * @param separator 分隔符
     * @return 字典标签
     */
    public static String getDictLabel(String dictType, String dictValue, String separator) {
        StringBuilder propertyString = new StringBuilder();
        List<Dict> datas = getDictCache(dictType);

        if (StrUtil.containsAny(separator, dictValue) && datas != null) {
            for (Dict dict : datas) {
                for (String value : dictValue.split(separator)) {
                    if (value.equals(dict.getDictValue())) {
                        propertyString.append(dict.getDictLabel()).append(separator);
                        break;
                    }
                }
            }
        } else {
            if (datas != null) {
                for (Dict dict : datas) {
                    if (dictValue.equals(dict.getDictValue())) {
                        return dict.getDictLabel();
                    }
                }
            }
        }
        //        StrUtil.stripEnd(propertyString.toString(), separator);
        return null;
    }

    /**
     * 根据字典类型和字典标签获取字典值
     *
     * @param dictType  字典类型
     * @param dictLabel 字典标签
     * @param separator 分隔符
     * @return 字典值
     */
    public static String getDictValue(String dictType, String dictLabel, String separator) {
        StringBuilder propertyString = new StringBuilder();
        List<Dict> datas = getDictCache(dictType);

        if (StrUtil.containsAny(separator, dictLabel) && datas != null) {
            for (Dict dict : datas) {
                for (String label : dictLabel.split(separator)) {
                    if (label.equals(dict.getDictLabel())) {
                        propertyString.append(dict.getDictValue()).append(separator);
                        break;
                    }
                }
            }
        } else {
            if (datas != null) {
                for (Dict dict : datas) {
                    if (dictLabel.equals(dict.getDictLabel())) {
                        return dict.getDictValue();
                    }
                }
            }
        }
        //        StrUtils.stripEnd(propertyString.toString(), separator)
        return null;
    }

    /**
     * 删除指定字典缓存
     *
     * @param key 字典键
     */
    public static void removeDictCache(String key) {
        SpringUtil.getBean(Redis.class).del(getCacheKey(key));
    }

    /**
     * 清空字典缓存
     */
    public static void clearDictCache() {
        Collection<String> keys = SpringUtil.getBean(Redis.class).keys("sys_dict:*");
        SpringUtil.getBean(Redis.class).del(keys.toArray(new String[0]));
    }

    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    public static String getCacheKey(String configKey) {
        return "sys_dict:" + configKey;
    }
}
